from panda3d.core import CullFaceAttrib
from panda3d.bullet import BulletBoxShape, BulletRigidBodyNode

class Box:

    def __init__(self, world):
        self._world = world
        shape = BulletBoxShape((.5, .5, .5))
        self.node = BulletRigidBodyNode('box')
        self.node.add_shape(shape)
        self._np = render.attach_new_node(self.node)
        self._np.set_pos(0, 0, 1)
        world.attach_rigid_body(self.node)
        model = loader.load_model('assets/gltf/box/box.gltf')
        model.flatten_light()
        model.reparent_to(self._np)
        self._set_outline_model()
        self._start_drag_pos = None
        self._prev_rot_info = None
        taskMgr.add(self.on_frame, 'on_frame')

    def _set_outline_model(self):
        self._outline_model = loader.load_model('assets/gltf/box/box.gltf')
        clockw = CullFaceAttrib.MCullClockwise
        self._outline_model.set_attrib(CullFaceAttrib.make(clockw))
        self._outline_model.reparent_to(self._np)
        self._outline_model.set_scale(-1.08, -1.08, -1.08)
        self._outline_model.set_light_off()
        self._outline_model.set_color(.4, .4, .4, 1)
        self._outline_model.set_color_scale(.4, .4, .4, 1)
        self._outline_model.hide()

    def on_frame(self, task):
        self._np.set_y(0)
        return task.cont

    def play(self):
        self._world.remove_rigid_body(self.node)
        self.node.set_mass(1)
        self._world.attach_rigid_body(self.node)

    def on_click_l(self, pos):
        self._start_drag_pos = pos, self._np.get_pos()
        loader.load_sfx('assets/audio/sfx/grab.ogg').play()

    def on_click_r(self, pos):
        self._prev_rot_info = pos, self._np.get_pos(), self._np.get_r()
        loader.load_sfx('assets/audio/sfx/grab.ogg').play()

    def on_release(self):
        if self._start_drag_pos or self._prev_rot_info:
            loader.load_sfx('assets/audio/sfx/release.ogg').play()
        self._start_drag_pos = self._prev_rot_info = None

    def on_mouse_on(self):
        self._outline_model.show()

    def on_mouse_off(self):
        if self._start_drag_pos or self._prev_rot_info: return
        self._outline_model.hide()

    def on_mouse_move(self, pos):
        if self._start_drag_pos:
            d_pos =  pos - self._start_drag_pos[0]
            self._np.set_pos(self._start_drag_pos[1] + d_pos)
        if self._prev_rot_info:
            start_vec = self._prev_rot_info[0] - self._prev_rot_info[1]
            curr_vec = pos - self._prev_rot_info[1]
            d_angle = curr_vec.signed_angle_deg(start_vec, (0, -1, 0))
            self._np.set_r(self._prev_rot_info[2] + d_angle)
            self._prev_rot_info = pos, self._np.get_pos(), self._np.get_r()
